#!/usr/bin/env python
# encoding: utf-8

from __future__ import absolute_import
import numpy as np
from scipy import integrate
from six.moves import range

gamma = 1.4
gamma1 = gamma - 1.
x0=0.5; y0=0.; r0=0.2
xshock = 0.2
pinf=5.

def inrad(y,x):
    return (np.sqrt((x-x0)**2+(y-y0)**2)<r0)

def ycirc(x,ymin,ymax):
    if r0**2>((x-x0)**2):
        return max(min(y0 + np.sqrt(r0**2-(x-x0)**2),ymax) - ymin,0.)
    else:
        return 0

def qinit(state,rhoin=0.1):
    r"""
    Initialize data with a shock at x=xshock and a low-density bubble (of density rhoin)
    centered at (x0,y0) with radius r0.
    """
    rhoout = 1.
    pout   = 1.
    pin    = 1.

    rinf = (gamma1 + pinf*(gamma+1.))/ ((gamma+1.) + gamma1*pinf)
    vinf = 1./np.sqrt(gamma) * (pinf - 1.) / np.sqrt(0.5*((gamma+1.)/gamma) * pinf+0.5*gamma1/gamma)
    einf = 0.5*rinf*vinf**2 + pinf/gamma1
    
    x =state.grid.x.center
    y =state.grid.y.center
    Y,X = np.meshgrid(y,x)
    r = np.sqrt((X-x0)**2 + (Y-y0)**2)

    #First set the values for the cells that don't intersect the bubble boundary
    state.q[0,:,:] = rinf*(X<xshock) + rhoin*(r<=r0) + rhoout*(r>r0)
    state.q[1,:,:] = rinf*vinf*(X<xshock)
    state.q[2,:,:] = 0.
    state.q[3,:,:] = einf*(X<xshock) + (pin*(r<=r0) + pout*(r>r0))/gamma1
    state.q[4,:,:] = 1.*(r<=r0)

    #Now average for the cells on the edge of the bubble
    d2 = np.linalg.norm(state.grid.d)/2.
    dx = state.grid.d[0]
    dy = state.grid.d[1]
    dx2 = state.grid.d[0]/2.
    dy2 = state.grid.d[1]/2.
    for i in range(state.q.shape[1]):
        for j in range(state.q.shape[2]):
            ydown = y[j]-dy2
            yup   = y[j]+dy2
            if abs(r[i,j]-r0)<d2:
                infrac,abserr = integrate.quad(ycirc,x[i]-dx2,x[i]+dx2,args=(ydown,yup),epsabs=1.e-8,epsrel=1.e-5)
                infrac=infrac/(dx*dy)
                state.q[0,i,j] = rhoin*infrac + rhoout*(1.-infrac)
                state.q[3,i,j] = (pin*infrac + pout*(1.-infrac))/gamma1
                state.q[4,i,j] = 1.*infrac


def auxinit(state):
    """
    aux[0,i,j] = y-coordinate of cell center for cylindrical source terms
    """
    y=state.grid.y.center
    for j,ycoord in enumerate(y):
        state.aux[0,:,j] = ycoord


def shockbc(state,dim,t,qbc,mbc):
    """
    Incoming shock at left boundary.
    """
    if dim.nstart == 0:

        rinf = (gamma1 + pinf*(gamma+1.))/ ((gamma+1.) + gamma1*pinf)
        vinf = 1./np.sqrt(gamma) * (pinf - 1.) / np.sqrt(0.5*((gamma+1.)/gamma) * pinf+0.5*gamma1/gamma)
        einf = 0.5*rinf*vinf**2 + pinf/gamma1

        for i in range(mbc):
            qbc[0,i,...] = rinf
            qbc[1,i,...] = rinf*vinf
            qbc[2,i,...] = 0.
            qbc[3,i,...] = einf
            qbc[4,i,...] = 0.

def dq_Euler_radial(solver,state,dt):
    """
    Geometric source terms for Euler equations with radial symmetry.
    Integrated using a 2-stage, 2nd-order Runge-Kutta method.
    This is a SharpClaw-style source term routine.
    """
    
    ndim = 2

    q   = state.q
    aux = state.aux

    rad = aux[0,:,:]

    rho = q[0,:,:]
    u   = q[1,:,:]/rho
    v   = q[2,:,:]/rho
    press  = gamma1 * (q[3,:,:] - 0.5*rho*(u**2 + v**2))

    dq = np.empty(q.shape)

    dq[0,:,:] = -dt*(ndim-1)/rad * q[2,:,:]
    dq[1,:,:] = -dt*(ndim-1)/rad * rho*u*v
    dq[2,:,:] = -dt*(ndim-1)/rad * rho*v*v
    dq[3,:,:] = -dt*(ndim-1)/rad * v * (q[3,:,:] + press)
    dq[4,:,:] = 0

    return dq

def step_Euler_radial(solver,state,dt):
    """
    Geometric source terms for Euler equations with radial symmetry.
    Integrated using a 2-stage, 2nd-order Runge-Kutta method.
    This is a Clawpack-style source term routine.
    """
    
    dt2 = dt/2.
    ndim = 2

    aux=state.aux
    q = state.q

    rad = aux[0,:,:]

    rho = q[0,:,:]
    u   = q[1,:,:]/rho
    v   = q[2,:,:]/rho
    press  = gamma1 * (q[3,:,:] - 0.5*rho*(u**2 + v**2))

    qstar = np.empty(q.shape)

    qstar[0,:,:] = q[0,:,:] - dt2*(ndim-1)/rad * q[2,:,:]
    qstar[1,:,:] = q[1,:,:] - dt2*(ndim-1)/rad * rho*u*v
    qstar[2,:,:] = q[2,:,:] - dt2*(ndim-1)/rad * rho*v*v
    qstar[3,:,:] = q[3,:,:] - dt2*(ndim-1)/rad * v * (q[3,:,:] + press)

    rho = qstar[0,:,:]
    u   = qstar[1,:,:]/rho
    v   = qstar[2,:,:]/rho
    press  = gamma1 * (qstar[3,:,:] - 0.5*rho*(u**2 + v**2))

    q[0,:,:] = q[0,:,:] - dt*(ndim-1)/rad * qstar[2,:,:]
    q[1,:,:] = q[1,:,:] - dt*(ndim-1)/rad * rho*u*v
    q[2,:,:] = q[2,:,:] - dt*(ndim-1)/rad * rho*v*v
    q[3,:,:] = q[3,:,:] - dt*(ndim-1)/rad * v * (qstar[3,:,:] + press)


def shockbubble(use_petsc=False,iplot=False,htmlplot=False,outdir='./_output',solver_type='classic'):
    """
    Solve the Euler equations of compressible fluid dynamics.
    This example involves a bubble of dense gas that is impacted by a shock.
    """

    if use_petsc:
        import petclaw as pyclaw
    else:
        import pyclaw

    if solver_type=='sharpclaw':
        solver = pyclaw.SharpClawSolver2D()
        solver.dq_src=dq_Euler_radial
    else:
        solver = pyclaw.ClawSolver2D()
        solver.dim_split = 0
        solver.order_trans = 2
        solver.limiters = [4,4,4,4,2]
        solver.step_src=step_Euler_radial

    solver.mwaves = 5
    solver.bc_lower[0]=pyclaw.BC.custom
    solver.bc_upper[0]=pyclaw.BC.outflow
    solver.bc_lower[1]=pyclaw.BC.reflecting
    solver.bc_upper[1]=pyclaw.BC.outflow

    #Aux variable in ghost cells doesn't matter
    solver.aux_bc_lower[0]=pyclaw.BC.outflow
    solver.aux_bc_upper[0]=pyclaw.BC.outflow
    solver.aux_bc_lower[1]=pyclaw.BC.outflow
    solver.aux_bc_upper[1]=pyclaw.BC.outflow

    # Initialize grid
    mx=320; my=80
    x = pyclaw.Dimension('x',0.0,2.0,mx)
    y = pyclaw.Dimension('y',0.0,0.5,my)
    grid = pyclaw.Grid([x,y])
    meqn = 5
    maux=8
    state = pyclaw.State(grid,meqn,maux)

    state.aux_global['gamma']= gamma
    state.aux_global['gamma1']= gamma1

    qinit(state)
    auxinit(state)

    solver.user_bc_lower=shockbc

    claw = pyclaw.Controller()
    claw.tfinal = 0.75
    claw.solution = pyclaw.Solution(state)
    claw.solver = solver
    claw.nout = 10
    claw.outdir = outdir

    # Solve
    status = claw.run()

    if htmlplot:  pyclaw.plot.html_plot(outdir=outdir)
    if iplot:     pyclaw.plot.interactive_plot(outdir=outdir)

    return claw.solution.q

if __name__=="__main__":
    from pyclaw.util import run_app_from_main
    output = run_app_from_main(shockbubble)



#!/usr/bin/env python

r"""Multilayer Shallow Water Test Case

To create new regression data use
    `python regression_tests.py True`
"""

from __future__ import absolute_import
import os
import sys
import unittest

import numpy

import clawpack.geoclaw.test as test
import clawpack.geoclaw.topotools as topotools


def transform_p2c(x, y, x0, y0, theta):
    return ( x*numpy.cos(theta) + y*numpy.sin(theta) - x0,
            -x*numpy.sin(theta) + y*numpy.cos(theta) - y0)

# Bathymetry
def bathy_step(x, y, location=0.15, angle=0.0, left=-1.0, right=-0.2):
    x_c,y_c = transform_p2c(x, y, location, 0.0, angle)
    return ((x_c <= 0.0) * left
          + (x_c >  0.0) * right)


class MultilayerTest(test.GeoClawRegressionTest):

    r"""Multilayer plane-wave regression test for GeoClaw

    initial condition angle = numpy.pi / 4.0
    bathy_angle = numpy.pi / 8.0

    """

    def setUp(self):

        super(MultilayerTest, self).setUp()

        # Make topography
        topo_func = lambda x, y: bathy_step(x, y, location=0.15,
                                                  angle=numpy.pi / 8.0,
                                                  left=-1.0, right=-0.2)
        topo = topotools.Topography(topo_func=topo_func)
        topo.x = numpy.linspace(-1.16, 2.16, 166)
        topo.y = numpy.linspace(-1.16, 2.16, 166)
        topo.write(os.path.join(self.temp_path, "jump_topo.topotype2"))

    def runTest(self, save=False):
        r"""Test multi-layer basic plane-waves."""

        # Load and write data, change init-condition's starting angle
        self.load_rundata()
        self.rundata.qinit_data.angle = numpy.pi / 4.0
        self.write_rundata_objects()

        # Run code and check surface heights
        self.run_code()
        self.check_gauges(save=save, gauge_id=0, indices=(6, 7), atol=1e-5)
        self.check_gauges(save=save, gauge_id=1, indices=(6, 7), atol=1e-5)
        self.check_gauges(save=save, gauge_id=2, indices=(6, 7), atol=1e-5)
        self.check_gauges(save=save, gauge_id=3, indices=(6, 7), atol=1e-5)
        self.check_gauges(save=save, gauge_id=4, indices=(6, 7), atol=1e-5)

        # If we have gotten here then we do not need to copy the run results
        self.success = True


if __name__ == "__main__":
    if len(sys.argv) > 1:
        if bool(sys.argv[1]):
            # Fake the setup and save out output
            test = MultilayerTest()
            try:
                test.setUp()
                test.runTest(save=True)
            finally:
                test.tearDown()
            sys.exit(0)
    unittest.main()



#!/usr/bin/env python
# encoding: utf-8
r"""
Two-dimensional variable-coefficient acoustics on a mapped grid
===============================================================

Solve the variable-coefficient acoustics equations in 2D:

.. math:: 
    p_t + K(x,y) (u_x + v_y) & = 0 \\ 
    u_t + p_x / \rho(x,y) & = 0 \\
    v_t + p_y / \rho(x,y) & = 0.

Here p is the pressure, (u,v) is the velocity, :math:`K(x,y)` is the bulk modulus,
and :math:`\rho(x,y)` is the density.

This example shows how to solve a problem with variable coefficients on a mapped grid.
The domain contains circular inclusions with different acoustic properties.
"""
import numpy as np

# Circle radius, square radius, circle center:
# ((r1, r2), (x0, y0))
circles = ( ((0.15,0.204),( 0.45,0.3)),
            ((0.15,0.204),( 0.7, 0.75)))

impedance = (12.,10.)
sound_speed = (0.3,1.5)

def inclusion_mapping(xc, yc):
    """Apply mapping from square to circle for each circle specified.
       Leave rest of grid cartesian.
    """
    xp = xc+0.
    yp = yc+0.

    for circle in circles:
        circle_center = circle[1]
        r1 = circle[0][0]
        r2 = circle[0][1]
    
        x0, y0 = circle_center
        xdist = np.abs(xc-x0)
        ydist = np.abs(yc-y0)
        insquare = np.where(np.maximum(xdist,ydist)<=r2) # Square section of grid to be deformed
        xc0 = (xc[insquare]-x0)/r2
        yc0 = (yc[insquare]-y0)/r2

        xc1 = np.abs(xc0)
        yc1 = np.abs(yc0)
        d = np.maximum(xc1, yc1)
        d = np.maximum(d,1.e-10)
        d = np.minimum(d, 0.99999)
        d1 = d*r2/np.sqrt(2)

        # Pick mapping
        #R = np.sqrt(2) * d1
        R = r1*np.ones(d1.shape)
        R = r1**2 / (r2*d)

        # Modify d1 and R outside circle to morph back to square:
        ij = np.where(d>(r1/r2))
        d1[ij] = r1/np.sqrt(2) + (d[ij]-r1/r2)*(r2-r1/np.sqrt(2))/(1.-r1/r2)
        R[ij] = r1 * ((1.-r1/r2) / (1.-d[ij]))**(r2/r1 + 0.5)

        xp2 = d1/d * xc1
        yp2 = d1/d * yc1
        center = d1 - np.sqrt(R**2 - d1**2)

        ij = np.where(xc1>=yc1)
        xp2[ij] = center[ij] + np.sqrt(R[ij]**2 - yp2[ij]**2)

        ij = np.where(yc1>=xc1)
        yp2[ij] = center[ij] + np.sqrt(R[ij]**2 - xp2[ij]**2)

        xp2 = np.sign(xc0) * xp2
        yp2 = np.sign(yc0) * yp2

        xp[insquare] = x0 + xp2
        yp[insquare] = y0 + yp2
    
    return xp, yp

def compute_geometry(grid):
    r"""Computes
        a_x
        a_y
        length_ratio_left
        b_x
        b_y
        length_ratio_bottom
        cell_area
    """

    dx, dy = grid.delta
    area_min = 1.e6
    area_max = 0.0

    x_corners, y_corners = grid.p_nodes

    lower_left_y, lower_left_x = y_corners[:-1,:-1], x_corners[:-1,:-1]
    upper_left_y, upper_left_x = y_corners[:-1,1: ], x_corners[:-1,1: ]
    lower_right_y, lower_right_x = y_corners[1:,:-1], x_corners[1:,:-1]
    upper_right_y, upper_right_x = y_corners[1:,1: ], x_corners[1:,1: ]

    a_x =   upper_left_y - lower_left_y  #upper left and lower left
    a_y = -(upper_left_x - lower_left_x)
    anorm = np.sqrt(a_x**2 + a_y**2)
    a_x, a_y = a_x/anorm, a_y/anorm
    length_ratio_left = anorm/dy

    b_x = -(lower_right_y - lower_left_y)  #lower right and lower left
    b_y =   lower_right_x - lower_left_x
    bnorm = np.sqrt(b_x**2 + b_y**2)
    b_x, b_y = b_x/bnorm, b_y/bnorm
    length_ratio_bottom = bnorm/dx

    area = 0*grid.c_centers[0]
    area += 0.5 * (lower_left_y+upper_left_y)*(upper_left_x-lower_left_x)
    area += 0.5 * (upper_left_y+upper_right_y)*(upper_right_x-upper_left_x)
    area += 0.5 * (upper_right_y+lower_right_y)*(lower_right_x-upper_right_x)
    area += 0.5 * (lower_right_y+lower_left_y)*(lower_left_x-lower_right_x)
    area = area/(dx*dy)
    area_min = min(area_min, np.min(area))
    area_max = max(area_max, np.max(area))

    return a_x, a_y, length_ratio_left, b_x, b_y, length_ratio_bottom, area

def incoming_square_wave(state,dim,t,qbc,auxbc,num_ghost):
    """
    Incoming square wave at left boundary.
    """
    if t<0.05:
        s = 1.0
    else:
        s = 0.0

    for i in range(num_ghost):
        reflect_ind = 2*num_ghost - i - 1
        alpha = auxbc[0,i,:]
        beta  = auxbc[1,i,:]
        u_normal = alpha*qbc[1,reflect_ind,:] + beta*qbc[2,reflect_ind,:]
        u_tangential = -beta*qbc[1,reflect_ind,:] + alpha*qbc[2,reflect_ind,:]
        u_normal = 2.0*s - u_normal
        qbc[0,i,:] = qbc[0,reflect_ind,:]
        qbc[1,i,:] = alpha*u_normal - beta*u_tangential
        qbc[2,i,:] = beta*u_normal + alpha*u_tangential


def setup(kernel_language='Fortran', use_petsc=False, outdir='./_output', 
          solver_type='classic', time_integrator='SSP104', lim_type=2, 
          num_output_times=20, disable_output=False, num_cells=200):
    from clawpack import riemann

    if use_petsc:
        import clawpack.petclaw as pyclaw
    else:
        from clawpack import pyclaw

    riemann_solver = riemann.acoustics_mapped_2D

    if solver_type=='classic':
        solver=pyclaw.ClawSolver2D(riemann_solver)
        solver.dimensional_split=False
        solver.limiters = pyclaw.limiters.tvd.MC
    elif solver_type=='sharpclaw':
        solver=pyclaw.SharpClawSolver2D(riemann_solver)
        solver.time_integrator=time_integrator

    solver.bc_lower[0]=pyclaw.BC.custom
    solver.user_bc_lower = incoming_square_wave
    solver.bc_upper[0]=pyclaw.BC.extrap
    solver.bc_lower[1]=pyclaw.BC.extrap
    solver.bc_upper[1]=pyclaw.BC.extrap

    solver.aux_bc_lower[0]=pyclaw.BC.extrap
    solver.aux_bc_upper[0]=pyclaw.BC.extrap
    solver.aux_bc_lower[1]=pyclaw.BC.extrap
    solver.aux_bc_upper[1]=pyclaw.BC.extrap

    x = pyclaw.Dimension(0.,1.0,num_cells,name='x')
    y = pyclaw.Dimension(0.,1.0,num_cells,name='y')
    domain = pyclaw.Domain([x,y])

    num_eqn = 3
    num_aux = 9 # geometry (7), impedance, sound speed
    state = pyclaw.State(domain,num_eqn,num_aux)
    state.grid.mapc2p = inclusion_mapping

    a_x, a_y, length_left, b_x, b_y, length_bottom, area = compute_geometry(state.grid)

    state.aux[0,:,:] = a_x
    state.aux[1,:,:] = a_y
    state.aux[2,:,:] = length_left
    state.aux[3,:,:] = b_x
    state.aux[4,:,:] = b_y
    state.aux[5,:,:] = length_bottom
    state.aux[6,:,:] = area
    state.index_capa = 6 # aux[6,:,:] holds the capacity function

    grid = state.grid
    xp, yp = grid.p_centers
    state.aux[7,:,:] = 1.0 # Impedance
    state.aux[8,:,:] = 1.0 # Sound speed

    for i, circle in enumerate(circles):
        # Set impedance and sound speed in each inclusion
        radius = circle[0][0]
        x0, y0 = circle[1]
        distance = np.sqrt( (xp-x0)**2 + (yp-y0)**2 )
        in_circle = np.where(distance <= radius)
        state.aux[7][in_circle] = impedance[i]
        state.aux[8][in_circle] = sound_speed[i]

    # Set initial condition
    state.q[0,:,:] = 0.
    state.q[1,:,:] = 0.
    state.q[2,:,:] = 0.

    claw = pyclaw.Controller()
    claw.keep_copy = True
    if disable_output:
        claw.output_format = None
    claw.solution = pyclaw.Solution(state,domain)
    claw.solver = solver
    claw.outdir = outdir
    claw.tfinal = 0.9
    claw.num_output_times = num_output_times
    claw.write_aux_init = True
    claw.setplot = setplot
    if use_petsc:
        claw.output_options = {'format':'binary'}

    return claw


def setplot(plotdata):
    """ 
    Plot solution using VisClaw.

    This example shows how to mark an internal boundary on a 2D plot.
    """ 

    from clawpack.visclaw import colormaps

    plotdata.clearfigures()  # clear any old figures,axes,items data
    plotdata.mapc2p = inclusion_mapping
    
    # Figure for pressure
    plotfigure = plotdata.new_plotfigure(name='Pressure', figno=0)

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.title = 'Pressure'
    plotaxes.scaled = True      # so aspect ratio is 1
    plotaxes.afteraxes = plot_circles

    # Set up for item on these axes:
    plotitem = plotaxes.new_plotitem(plot_type='2d_contour')
    plotitem.contour_nlevels = 100
    plotitem.contour_min = -2.51
    plotitem.contour_max = 2.51
    plotitem.plot_var = 0
    
    # Figure for x-velocity plot
    plotfigure = plotdata.new_plotfigure(name='x-Velocity', figno=1)

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.title = 'u'
    plotaxes.afteraxes = plot_circles

    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    plotitem.plot_var = 1
    plotitem.pcolor_cmap = 'RdBu'
    plotitem.add_colorbar = True
    plotitem.pcolor_cmin = -1.0
    plotitem.pcolor_cmax=   1.0
    
    return plotdata

def plot_circles(current_data):
    import matplotlib.pyplot as plt
    ax = plt.gca()
    for circle in circles:
        x0, y0 = circle[1]
        radius = circle[0][0]
        circle1=plt.Circle((x0,y0),radius,color='k',fill=False,lw=3)
        ax.add_artist(circle1)


if __name__=="__main__":
    import sys
    from clawpack.pyclaw.util import run_app_from_main
    output = run_app_from_main(setup,setplot)



#!/usr/bin/env python
# encoding: utf-8
r"""
Test suite for forestclaw.io
"""

from __future__ import absolute_import
from __future__ import print_function

import os
import shutil
import tempfile
import shutil

import numpy
import nose

from .. import geometry
from clawpack.pyclaw.solution import Solution
from clawpack.pyclaw.state import State


def test_forestclaw_input():
    """Simple test to read in a ForestClaw ASCII file"""

    # Create test solution
    x = geometry.Dimension(0.0, 1.0, 100, name='x')
    state = State(geometry.Patch(x), 1)
    state.q = numpy.zeros((1, x.num_cells))
    sol = Solution(state, geometry.Domain(x))

    # Test specific extensions to Patch
    sol.domain.patches[0].block_number = 2
    sol.domain.patches[0].mpi_rank = 2

    # Create temporary directory to write to and read from
    try:
        temp_path = tempfile.mkdtemp()

        # Real test
        sol.write(0, path=temp_path)
        read_sol = Solution(0, path=temp_path,
                                       file_format='forestclaw')
        nose.tools.assert_equal(sol, read_sol,
                                "ForestClaw IO read/write test failed, " +
                                "solutions are not equal.")
    except:
        shutil.copy(os.path.join(temp_path, 'fort.q0000'), os.getcwd())
    finally:
        shutil.rmtree(temp_path)

if __name__ == '__main__':
    nose.main()



#!/usr/bin/env python
# encoding: utf-8
r"""
Two-dimensional p-system
==============================

Solve the two-dimensional generalization of the p-system:

.. math:: 
    \epsilon_t - u_x - v_y & = 0 \\
    \rho(x,y) u_t - \sigma(\epsilon,x,y)_x & = 0 \\
    \rho(x,y) v_t - \sigma(\epsilon,x,y)_y & = 0.

We take :math:`\sigma = e^{K(x,y)\epsilon} - 1`, and the
material coefficients :math:`\rho,K` vary in a checkerboard
pattern.  The resulting dynamics lead to solitary waves,
though much more resolution is needed in order to see them.

This example shows how to set an aux array, use a b4step function,
use gauges, compute output functionals, and restart a simulation
from a checkpoint.
"""

import numpy as np
from clawpack import riemann

def qinit(state,A,x0,y0,varx,vary):
    r""" Set initial conditions:
         Gaussian stress, zero velocities."""
    yy,xx = state.grid.c_centers
    stress=A*np.exp(-(xx-x0)**2/(2*varx)-(yy-y0)**2/(2*vary)) #sigma(@t=0)
    stress_rel=state.aux[2,:]
    K=state.aux[1,:]

    state.q[0,:,:]=np.where(stress_rel==1,1,0)*stress/K+np.where(stress_rel==2,1,0)*np.log(stress+1)/K
    state.q[1,:,:]=0; state.q[2,:,:]=0

def setaux(x,y, KA=1, KB=4, rhoA=1, rhoB=4, stress_rel=2):
    r"""Return an array containing the values of the material
        coefficients.

        aux[0,i,j] = rho(x_i, y_j)              (material density)
        aux[1,i,j] = K(x_i, y_j)                (bulk modulus)
        aux[2,i,j] = stress-strain relation type at (x_i, y_j)
    """
    alphax=0.5; deltax=1.
    alphay=0.5; deltay=1.
    
    medium_type = 'checkerboard'

    aux = np.empty((4,len(x),len(y)), order='F')
    if medium_type == 'checkerboard':
        # xfrac and yfrac are x and y relative to deltax and deltay resp.
        xfrac=x-np.floor(x/deltax)*deltax
        yfrac=y-np.floor(y/deltay)*deltay
        # create a meshgrid out of xfrac and yfrac
        [yf,xf]=np.meshgrid(yfrac,xfrac)
        # density 
        aux[0,:,:]=rhoA*(xf<=alphax*deltax)*(yf<=alphay*deltay)\
                  +rhoA*(xf >alphax*deltax)*(yf >alphay*deltay)\
                  +rhoB*(xf >alphax*deltax)*(yf<=alphay*deltay)\
                  +rhoB*(xf<=alphax*deltax)*(yf >alphay*deltay)
        #Young's modulus
        aux[1,:,:]=KA*(xf<=alphax*deltax)*(yf<=alphay*deltay)\
                  +KA*(xf >alphax*deltax)*(yf >alphay*deltay)\
                  +KB*(xf >alphax*deltax)*(yf<=alphay*deltay)\
                  +KB*(xf<=alphax*deltax)*(yf >alphay*deltay)
        # linearity of material
        aux[2,:,:]=stress_rel
    elif medium_type == 'sinusoidal' or medium_type == 'smooth_checkerboard':
        [yy,xx]=np.meshgrid(y,x)
        Amp_rho=np.abs(rhoA-rhoB)/2; offset_p=(rhoA+rhoB)/2
        Amp_K=np.abs(KA-KB)/2; offset_E=(KA+KB)/2
        if medium_type == 'sinusoidal':
            frec_x=2*np.pi/deltax; frec_y=2*np.pi/deltay
            fun=np.sin(frec_x*xx)*np.sin(frec_y*yy)
        else:
            sharpness=10
            fun_x=xx*0; fun_y=yy*0
            for i in range(0,1+int(np.ceil((x[-1]-x[0])/(deltax*0.5)))):
                fun_x=fun_x+(-1)**i*np.tanh(sharpness*(xx-deltax*i*0.5))
            for i in range(0,1+int(np.ceil((y[-1]-y[0])/(deltay*0.5)))):
                fun_y=fun_y+(-1)**i*np.tanh(sharpness*(yy-deltay*i*0.5))
            fun=fun_x*fun_y
        aux[0,:,:]=Amp_rho*fun+offset_p
        aux[1,:,:]=Amp_K*fun+offset_E
        aux[2,:,:]=stress_rel
    return aux

def b4step(solver,state):
    r"""Put in aux[3,:,:] the value of q[0,:,:] (eps). 
        This is required in rptpv.f.
        Only used by classic (not SharpClaw).
    """
    state.aux[3,:,:] = state.q[0,:,:]

def compute_stress(state):
    """ Compute stress from strain and store in state.p."""
    K=state.aux[1,:,:]
    stress_rel=state.aux[2,:,:]
    eps=state.q[0,:,:]
    state.p[0,:,:] = np.where(stress_rel==1,1,0) * K*eps \
                    +np.where(stress_rel==2,1,0) * (np.exp(eps*K)-1) \
 

def total_energy(state):
    rho = state.aux[0,:,:]; K = state.aux[1,:,:]
    
    u = state.q[1,:,:]/rho
    v = state.q[2,:,:]/rho
    kinetic=rho * (u**2 + v**2)/2.

    eps = state.q[0,:,:]
    sigma = np.exp(K*eps) - 1.
    potential = (sigma-np.log(sigma+1.))/K

    dx=state.grid.delta[0]; dy=state.grid.delta[1]
    
    state.F[0,:,:] = (potential+kinetic)*dx*dy 

def gauge_stress(q,aux):
    p = np.exp(q[0]*aux[1])-1
    return [p,10*p]


def setup(kernel_language='Fortran',
              use_petsc=False,outdir='./_output',solver_type='classic',
              disable_output=False, cells_per_layer=30, tfinal=18.):

    if use_petsc:
        import clawpack.petclaw as pyclaw
    else:
        from clawpack import pyclaw

    # material parameters
    KA=1.;   rhoA=1.
    KB=4.;   rhoB=4.
    stress_rel=2;

    # Domain
    x_lower=0.25; x_upper=20.25
    y_lower=0.25; y_upper=20.25
    # cells per layer
    mx=int((x_upper-x_lower)*cells_per_layer)
    my=int((y_upper-y_lower)*cells_per_layer)
    # Initial condition parameters
    initial_amplitude=10.
    x0=0.25 # Center of initial perturbation
    y0=0.25 # Center of initial perturbation
    varx=0.5; vary=0.5 # Width of initial perturbation

    num_output_times = 10

    if solver_type == 'classic':
        solver = pyclaw.ClawSolver2D(riemann.psystem_2D)
        solver.dimensional_split=False
        solver.cfl_max = 0.9
        solver.cfl_desired = 0.8
        solver.limiters = pyclaw.limiters.tvd.superbee
    elif solver_type=='sharpclaw':
        solver = pyclaw.SharpClawSolver2D(riemann.psystem_2D)

    if kernel_language != 'Fortran':
        raise Exception('Unrecognized value of kernel_language for 2D psystem')

    # Boundary conditions
    solver.bc_lower[0] = pyclaw.BC.wall
    solver.bc_upper[0] = pyclaw.BC.extrap
    solver.bc_lower[1] = pyclaw.BC.wall
    solver.bc_upper[1] = pyclaw.BC.extrap
    solver.aux_bc_lower[0] = pyclaw.BC.wall
    solver.aux_bc_upper[0] = pyclaw.BC.extrap
    solver.aux_bc_lower[1] = pyclaw.BC.wall
    solver.aux_bc_upper[1] = pyclaw.BC.extrap

    solver.fwave = True
    solver.before_step = b4step

    #controller
    claw = pyclaw.Controller()
    claw.tfinal = tfinal
    claw.solver = solver
    claw.outdir = outdir
    
    # restart options
    restart_from_frame = None

    if restart_from_frame is None:
        x = pyclaw.Dimension(x_lower,x_upper,mx,name='x')
        y = pyclaw.Dimension(y_lower,y_upper,my,name='y')
        domain = pyclaw.Domain([x,y])
        num_eqn = 3
        num_aux = 4
        state = pyclaw.State(domain,num_eqn,num_aux)
        state.mF = 1
        state.mp = 1

        grid = state.grid
        state.aux = setaux(grid.x.centers,grid.y.centers,KA,KB,rhoA,rhoB,stress_rel)
        #Initial condition
        qinit(state,initial_amplitude,x0,y0,varx,vary)

        claw.solution = pyclaw.Solution(state,domain)
        claw.num_output_times = num_output_times

    else:
        claw.solution = pyclaw.Solution(restart_from_frame, format='petsc',read_aux=False)
        claw.solution.state.mp = 1
        grid = claw.solution.domain.grid
        claw.solution.state.aux = setaux(grid.x.centers,grid.y.centers)
        claw.num_output_times = num_output_times - restart_from_frame
        claw.start_frame = restart_from_frame

    #claw.p_function = p_function
    if disable_output:
        claw.output_format = None
    claw.compute_F = total_energy
    claw.compute_p = compute_stress
    claw.write_aux_init = False

    grid.add_gauges([[0.25,0.25],[17.85,1.25],[3.25,18.75],[11.75,11.75]])
    solver.compute_gauge_values = gauge_stress
    state.keep_gauges = True
    claw.setplot = setplot
    claw.keep_copy = True

    return claw

#--------------------------
def setplot(plotdata):
#--------------------------
    """ 
    Specify what is to be plotted at each frame.
    Input:  plotdata, an instance of visclaw.data.ClawPlotData.
    Output: a modified version of plotdata.
    """ 
    from clawpack.visclaw import colormaps

    plotdata.clearfigures()  # clear any old figures,axes,items data
    
    # Figure for strain
    plotfigure = plotdata.new_plotfigure(name='Stress', figno=0)

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.title = 'Strain'
    plotaxes.xlimits = [0.,20.]
    plotaxes.ylimits = [0.,20.]
    plotaxes.scaled = True

    # Set up for item on these axes:
    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    plotitem.plot_var = stress
    plotitem.pcolor_cmap = colormaps.yellow_red_blue
    plotitem.add_colorbar = True
    
    return plotdata

def stress(current_data):    
    import numpy as np
    aux = setaux(current_data.x[:,0],current_data.y[0,:])
    q = current_data.q
    return np.exp(aux[1,...]*q[0,...])-1.


if __name__=="__main__":
    from clawpack.pyclaw.util import run_app_from_main
    output = run_app_from_main(setup,setplot)





import os, sys
from clawpack.visclaw import geoplot
from clawpack.geoclaw.nonuniform_grid_tools import make_mapc2p
import numpy


xlimits = [-160,40]

fname_celledges = os.path.abspath('celledges.data')

outdir2 = None  # set to compare different models

def setplot(plotdata):

    plotdata.clearfigures()

    def set_dry_tol(current_data):
        current_data.user['dry_tolerance'] = 1e-6

    plotdata.beforeframe = set_dry_tol

    outdir1 = plotdata.outdir
    mapc2p1, mx_edge, xp_edge = make_mapc2p(fname_celledges)


    def fix_layout(current_data):
        from pylab import tight_layout
        tight_layout()


    plotfigure = plotdata.new_plotfigure(name='domain', figno=0)
    plotfigure.figsize = (8,7)
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.axescmd = 'subplot(311)'
    plotaxes.xlimits = xlimits
    plotaxes.ylimits = [-0.3,0.6]
    plotaxes.grid = True
    plotaxes.title = 'Surface displacement'

    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    plotitem.plot_var = geoplot.surface
    plotitem.color = 'b'
    #plotitem.plotstyle = '-+'
    plotitem.MappedGrid = True
    plotitem.mapc2p = mapc2p1

    if outdir2 is not None:
        plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
        plotitem.outdir = outdir2
        plotitem.plot_var = geoplot.surface
        plotitem.color = 'r'
        #plotitem.plotstyle = '-+'
        plotitem.MappedGrid = True
        plotitem.mapc2p = mapc2p1

    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    #plotitem.show = False
    plotitem.plot_var = geoplot.topo
    plotitem.color = 'g'
    plotitem.MappedGrid = True
    plotitem.mapc2p = mapc2p1

    plotaxes = plotfigure.new_plotaxes()
    plotaxes.axescmd = 'subplot(312)'
    plotaxes.xlimits = xlimits
    plotaxes.ylimits = [-1,1]
    plotaxes.grid = True
    plotaxes.title = 'Velocity'
    plotitem.MappedGrid = True
    plotitem.mapc2p = mapc2p1

    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    plotitem.plot_var = geoplot.u_velocity
    plotitem.color = 'b'
    plotitem.MappedGrid = True
    plotitem.mapc2p = mapc2p1

    if outdir2 is not None:
        plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
        plotitem.outdir = outdir2
        plotitem.plot_var = geoplot.u_velocity
        plotitem.color = 'r'
        plotitem.MappedGrid = True
        plotitem.mapc2p = mapc2p1

    plotaxes = plotfigure.new_plotaxes()
    plotaxes.axescmd = 'subplot(313)'
    plotaxes.xlimits = xlimits
    plotaxes.ylimits = [-5,2]
    plotaxes.grid = True
    plotaxes.title = 'Full depth'
    plotaxes.grid = True
    plotaxes.afteraxes = fix_layout
    plotitem.MappedGrid = True
    plotitem.mapc2p = mapc2p1
    
    plotitem = plotaxes.new_plotitem(plot_type='1d_fill_between')
    #plotitem.show = False
    plotitem.plot_var = geoplot.surface
    plotitem.plot_var2 = geoplot.topo
    plotitem.color = [.5,.5,1]
    plotitem.MappedGrid = True
    plotitem.mapc2p = mapc2p1

    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    #plotitem.show = False
    plotitem.plot_var = geoplot.topo
    plotitem.color = 'g'
    plotitem.MappedGrid = True
    plotitem.mapc2p = mapc2p1

    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    plotitem.plot_var = geoplot.surface
    plotitem.color = 'b'
    plotitem.MappedGrid = True
    plotitem.mapc2p = mapc2p1



    #-----------------------------------------
    # Figures for gauges
    #-----------------------------------------
    plotfigure = plotdata.new_plotfigure(name='q', figno=300, \
                                         type='each_gauge')
    plotfigure.figsize = (12,3)
    plotfigure.clf_each_gauge = True

    plotaxes = plotfigure.new_plotaxes()
    plotaxes.xlimits = 'auto'
    #plotaxes.ylimits = [-2,2]
    plotaxes.grid = True
    plotaxes.title = 'Surface elevation eta'

    def setxlim(current_data):
        from pylab import xlim
        gno = current_data.gaugeno
        if gno == 150:
            xlim(0,60)
        elif gno == 80:
            xlim(10,60)
        elif gno == 60:
            xlim(20,70)
        elif gno == 40:
            xlim(40,90)
        elif gno == 30:
            xlim(40,90)
        elif gno == 20:
            xlim(50,100)
        elif gno == 10:
            xlim(60,110)
    plotaxes.afteraxes = setxlim

    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    plotitem.plot_var = 2  # eta
    plotitem.plotstyle = 'b-'


    #-----------------------------------------

    # Parameters used only when creating html and/or latex hardcopy
    # e.g., via pyclaw.plotters.frametools.printframes:
    
    plotdata.printfigs = True                # print figures
    plotdata.print_format = 'png'            # file format
    plotdata.print_framenos = 'all'          # list of frames to print
    plotdata.print_gaugenos = 'all'          # list of gauges to print
    plotdata.print_fignos = 'all'            # list of figures to print
    plotdata.html = True                     # create html files of plots?
    plotdata.html_homelink = '../README.html'   # pointer for top of index
    plotdata.latex = True                    # create latex file of plots?
    plotdata.latex_figsperline = 2           # layout of plots
    plotdata.latex_framesperline = 1         # layout of plots
    plotdata.latex_makepdf = False           # also run pdflatex?
    plotdata.parallel = True                 # make multiple frame png's at once

    return plotdata




"""

Tools to download etopo topography/bathymetry data from NCEI (formerly NGDC).
See http://www.ngdc.noaa.gov/mgg/global/global.html

Note the new etopo1_download_nc is better to use than etopo1_download.
"""


from __future__ import absolute_import
from __future__ import print_function

def etopo1_download(xlimits, ylimits, dx=0.0166666666667, dy=None, \
        output_dir='.', file_name=None, force=False, verbose=True, \
        return_topo=False):

    """
    Create a url to download etopo1 topography from NCEI and
    save as a topo_type 3 file.  Uses the database described at
        http://www.ngdc.noaa.gov/mgg/global/global.html

    :Inputs:

    - *xlimits*: tuple (x1, x2) limits in longitude
      Must either have -180 <= x1 < x2 <= 180
           or 180 <= x1 < x2 <= 360
           or -360 <= x1 < x2 <= -180
      To download topo for a region spanning longitude 180, you must
      download two separate files, one on each side.

    - *ylimits*: tuple (y1, y2) limits in latitude
      Must have -90 <= y1 < y2 <= 90.

    - *dx*: resolution in x, default is 1./60. degree = 1 arcminute.
    - *dy*: resolution in y, default is dy = dx.
    - *output_dir*: directory to store file, default is '.'
    - *file_name*: name of file, default is constructed from xlimits,ylimits
    - *force*: if True, download even if the file already exists.
    - *verbose*: if True, print info from clawpack.clawutil.data.get_remote_file

    Note: New NGDC format gives cell-registered values, so shift the 
    values `xllcorner` and `yllcorner` to the specified corner.

    **To do:** Check whether it is possible to specify grid-registered
    values as implied at http://www.ngdc.noaa.gov/mgg/global/global.html

    The `nodata_value` line expected by GeoClaw is now missing,
    so add this in too.
    """

    from clawpack.geoclaw import util, topotools
    from clawpack.clawutil.data import get_remote_file
    import os
    from numpy import round

    format = '&format=aaigrid'   # topo_type 3

    if dy is None:
        dy = dx

    arcminute = 1/60.
    if abs(dx-arcminute)>1e-8 or abs(dy-arcminute)>1e-8:
        print('*** Warning: data may not be properly subsampled at')
        print('*** resolutions other than 1 arcminute, dx=dy=1/60.')

    x1,x2 = xlimits
    y1,y2 = ylimits

    if file_name is None:
        file_name = 'etopo1_%i_%i_%i_%i_%imin.asc' \
            % (int(round(x1)),int(round(x2)),int(round(y1)),int(round(y2)),\
              int(round(60*dx)))

    if (x1>=180) and (x1<x2) and (x2<=360):
        longitude_shift = -360.
    elif (x1>=-360) and (x1<x2) and (x2<=-180):
        longitude_shift = 360.
    else:
        longitude_shift = 0.
    x1 = x1 + longitude_shift
    x2 = x2 + longitude_shift

    if (x1<-180) or (x1>=x2) or (x2>180):
        raise ValueError("Require -180 <= x1 < x2 <= 180 or 180 <= x1 < x2 <=360")
    if (y1<-90) or (y1>=y2) or (y2>90):
        raise ValueError("Require -90 <= y1 < y2 <= 90")

    bbox = '&bbox=%1.4f,%1.4f,%1.4f,%1.4f' % (x1,y1,x2,y2)
    res = '&resx=%1.12f&resy=%1.12f' % (dx,dy)
    url = 'http://maps.ngdc.noaa.gov/mapviewer-support/wcs-proxy/wcs.groovy' \
            + '?request=getcoverage&version=1.0.0&service=wcs' \
            + '&coverage=etopo1&CRS=EPSG:4326' \
            + format + bbox + res

    file_path = os.path.join(output_dir,file_name)
    if os.path.exists(file_path) and (not force):
        print("Skipping download... file already exists: ",file_path)

    else:
        get_remote_file(url, output_dir=output_dir, file_name=file_name, \
                        verbose=verbose,force=force)

        x1 = x1 - longitude_shift   # shift back before writing header

        lines = open(file_path).readlines()
        if lines[2].split()[0] != 'xllcorner':
            print("*** Error downloading, check the file!")
        else:
            x1file = float(lines[2].split()[1])
            x2file = float(lines[3].split()[1])
            lines[2] = 'xllcorner    %1.12f\n' % x1file
            lines[3] = 'yllcorner    %1.12f\n' % x2file
            if 'nodata_value' not in lines[5]:
                lines = lines[:5] + ['nodata_value    -99999\n'] + lines[5:]
                print("Added nodata_value line")
            f = open(file_path,'w')
            f.writelines(lines)
            f.close()
        print("Created file: ",file_path)

    if return_topo:
        topo = topotools.Topography()
        topo.read(file_path, topo_type=3)
        return topo




import test_rp1_meqn2
import numpy

rho = 1.
bulk = 1.
test_rp1_meqn2.cparam.rho = rho
test_rp1_meqn2.cparam.bulk = bulk
test_rp1_meqn2.cparam.cc = numpy.sqrt(bulk/rho)
test_rp1_meqn2.cparam.zz = numpy.sqrt(bulk*rho)

ql = [2.,0.]
qr = [1.,0.]
wave,s,amdq,apdq = test_rp1_meqn2.rp1_driver.call_rp1(2,ql,qr)

print("s = ",s)
print("wave = ")
print(wave)



#!/usr/bin/env python
# encoding: utf-8
r"""
One-dimensional advection with variable velocity
================================================

Solve the non-conservative variable-coefficient advection equation
(also known as the color equation):

.. math:: q_t + u(x) q_x = 0.

Here q is the density of some quantity and u(x) is the velocity.
The velocity field used is

.. math:: u(x) = 2 + sin(2\pi x).

The boundary conditions are periodic.
The initial data get stretched and compressed as they move through the
fast and slow parts of the velocity field.
"""


import numpy as np

def qinit(state):

    # Initial Data parameters
    ic = 3
    beta = 100.
    gamma = 0.
    x0 = 0.3
    x1 = 0.7
    x2 = 0.9

    x =state.grid.x.centers
    
    # Gaussian
    qg = np.exp(-beta * (x-x0)**2) * np.cos(gamma * (x - x0))
    # Step Function
    qs = (x > x1) * 1.0 - (x > x2) * 1.0
    
    if   ic == 1: state.q[0,:] = qg
    elif ic == 2: state.q[0,:] = qs
    elif ic == 3: state.q[0,:] = qg + qs


def auxinit(state):
    # Initialize petsc Structures for aux
    xc=state.grid.x.centers
    state.aux[0,:] = np.sin(2.*np.pi*xc)+2
    

def setup(use_petsc=False,solver_type='classic',kernel_language='Python',outdir='./_output',
          weno_order=5,time_integrator='SSP104',rk_coeffs=None):
    from clawpack import riemann

    if use_petsc:
        import clawpack.petclaw as pyclaw
    else:
        from clawpack import pyclaw

    if solver_type=='classic':
        if kernel_language == 'Fortran':
            solver = pyclaw.ClawSolver1D(riemann.advection_color_1D)
        elif kernel_language=='Python': 
            solver = pyclaw.ClawSolver1D(riemann.vc_advection_1D_py.vc_advection_1D)
    elif solver_type=='sharpclaw':
        if kernel_language == 'Fortran':
            solver = pyclaw.SharpClawSolver1D(riemann.advection_color_1D)
        elif kernel_language=='Python': 
            solver = pyclaw.SharpClawSolver1D(riemann.vc_advection_1D_py.vc_advection_1D)
        solver.weno_order = weno_order
        solver.time_integrator = time_integrator
        if time_integrator == 'RK':
            solver.a, solver.b, solver.c = rk_coeffs
            solver.cfl_desired = 0.9
            solver.cfl_max = 1.0
    else: raise Exception('Unrecognized value of solver_type.')

    solver.kernel_language = kernel_language

    solver.limiters = pyclaw.limiters.tvd.MC
    solver.bc_lower[0] = 2
    solver.bc_upper[0] = 2
    solver.aux_bc_lower[0] = 2
    solver.aux_bc_upper[0] = 2

    xlower=0.0; xupper=1.0; mx=100
    x    = pyclaw.Dimension(xlower,xupper,mx,name='x')
    domain = pyclaw.Domain(x)
    num_aux=1
    num_eqn = 1
    state = pyclaw.State(domain,num_eqn,num_aux)

    qinit(state)
    auxinit(state)

    claw = pyclaw.Controller()
    claw.outdir = outdir
    if outdir is None:
        claw.output_format = None
    claw.solution = pyclaw.Solution(state,domain)
    claw.solver = solver

    claw.tfinal = 1.0
    claw.setplot = setplot
    claw.keep_copy = True
    
    return claw

#--------------------------
def setplot(plotdata):
#--------------------------
    """ 
    Specify what is to be plotted at each frame.
    Input:  plotdata, an instance of visclaw.data.ClawPlotData.
    Output: a modified version of plotdata.
    """ 
    plotdata.clearfigures()  # clear any old figures,axes,items data

    # Figure for q[0]
    plotfigure = plotdata.new_plotfigure(name='q', figno=1)

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.ylimits = [-.1,1.1]
    plotaxes.title = 'q'

    # Set up for item on these axes:
    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    plotitem.plot_var = 0
    plotitem.plotstyle = '-o'
    plotitem.color = 'b'
    plotitem.kwargs = {'linewidth':2,'markersize':5}
    
    return plotdata

 
if __name__=="__main__":
    from clawpack.pyclaw.util import run_app_from_main
    output = run_app_from_main(setup,setplot)



#!/usr/bin/env python
# encoding: utf-8
r"""
Variable coefficient 1D advection Riemann solver

.. math::
    q_t + u(x) q_x = 0.

Note that this is the color equation, not the conservative advection equation.
"""

import numpy as np

# Riemann solver constants
num_eqn = 1
num_waves = 1

def vc_advection_1D(q_l,q_r,aux_l,aux_r,problem_data):
    r"""Basic 1d advection riemann solver
    
    *aux(i)* should contain -
     - *u(x_i)* - (float) advection speed
    
    See :ref:`petclaw_rp` for more details.
    
    :Version: 1.0 (2010-10-10)
    """
    
    # Number of Riemann problems we are solving
    num_rp = q_l.shape[1]
    
    # Return values
    wave = np.empty( (num_eqn,num_waves,num_rp) )
    s = np.empty( (num_waves,num_rp) )
    amdq = np.zeros( (num_eqn,num_rp) )
    apdq = np.zeros( (num_eqn,num_rp) )
    
    wave[0,0,:] = q_r[0,:] - q_l[0,:]

    s[0,:] = aux_l[0,:]

    apdq[0,:] = (aux_l[0,:]>0)*s[0,:] * wave[0,0,:]
    amdq[0,:] = (aux_l[0,:]<0)*s[0,:] * wave[0,0,:]

    return wave, s, amdq, apdq




from __future__ import absolute_import
from __future__ import print_function
import glob, os, sys

try:
    from docutils.core import publish_string
except:
    print('*** Install docutils to convert README.rst')
    sys.exit(0)

if not os.path.isfile('README.rst'):
    print('*** README.rst file not found in %s' % os.getcwd())
    sys.exit()

if (sys.version_info > (3,0)):
    html_string=publish_string(open('README.rst').read(),writer_name='html',
                settings_overrides={'output_encoding': 'unicode'})
else:
    html_string=publish_string(open('README.rst').read(),writer_name='html')

make_files = glob.glob("Makefile")
f_files = glob.glob("*.f") + glob.glob("*.f90")
py_files = glob.glob("*.py")
m_files = glob.glob("*.m")
ipynb_files = glob.glob("*.ipynb")
out_dirs = glob.glob("_out*")   # not currently listed on html page
plot_dirs = glob.glob("_plots") # for gallery -- want only main _plots
#plot_dirs = glob.glob("_plot*")   # might want to list other plot directories


make_text = "\n"
for f in make_files:
    flink = f + '.html'
    if not os.path.isfile(flink): flink = f
    make_text = make_text + '<li><a href="%s">%s</a>\n' % (flink,f)

f_text = "\n"
for f in f_files:
    flink = f + '.html'
    if not os.path.isfile(flink): flink = f
    f_text = f_text + '<li><a href="%s">%s</a>\n' % (flink,f)

py_text = "\n"
for f in py_files:
    flink = f + '.html'
    if not os.path.isfile(flink): flink = f
    py_text = py_text + '<li><a href="%s">%s</a>\n' % (flink,f)

ipynb_text = "\n"
for f in ipynb_files:
    flink = f.replace('.ipynb', '.html')
    if os.path.isfile(flink): 
        ipynb_text = ipynb_text + '<li><a href="%s">%s</a>\n' % (flink,f)
    else:
        print('*** Found %s but missing %s' % (f,flink))

m_text = "\n"
for f in m_files:
    flink = f + '.html'
    if not os.path.isfile(flink): flink = f
    m_text = m_text + '<li><a href="%s">%s</a>\n' % (flink,f)

out_text = "\n"
for f in out_dirs:
    out_text = out_text + '<li><a href="%s">%s</a>\n' % (f,f)

plot_text = "\n"
for f in plot_dirs:
    if os.path.isfile("%s/_PlotIndex.html" % f):
        plot_text = plot_text + '<li><a href="%s/_PlotIndex.html">%s</a>\n' % (f,f)
    else:
        print("*** No _PlotIndex.html in ",f)


new_text = """
<h2>Files (html versions)</h2>
<ul>
%s
<p>
%s
<p>
%s
<p>
%s
<p>
%s
<p>
%s
</ul>
<p>
""" % (make_text,f_text,ipynb_text,py_text,m_text,plot_text)

# If README.rst contains a section labelled "Version", insert the list of
# files before this section.  Otherwise put list of files at end...

version_loc = html_string.find(r"<h1>Version</h1>")
end_body = html_string.find(r"</body>\n</html>")

if version_loc > -1:
    html_string = html_string[:version_loc] + new_text + html_string[version_loc:]
else:
    html_string = html_string[:end_body] + new_text + html_string[end_body:]

# Adjust size of section heading:
html_string = html_string.replace(r'<h1>Version</h1>','<h2>Version</h2>')

# Fix head for Clawpack style:
head_text = """
<link rel="icon" href="http://www.clawpack.org/_static/clawicon_new.ico" />
</head>
<body BGCOLOR="#FFFFE8" LINK="#7F0000" VLINK="#7F0000">
<font FACE="TREBUCHET MS,HELVETICA,ARIAL">
<a href="http://www.clawpack.org">
<IMG SRC="http://www.clawpack.org/_static/clawlogo_border.jpg" WIDTH="250" HEIGHT="70" 
VSPACE="0" HSPACE="0" ALT="CLAWPACK" BORDER="0" LOOP="0"> </a>
"""

i1 = html_string.find(r'</head>')
i2 = i1+15

html_string = html_string[:i1] + head_text + html_string[i2:]

open('README.html','w').write(html_string)




"""
Tools for defining and working with ruled rectangles.
"""

import numpy as np

class RuledRectangle(object):
    
    def __init__(self, fname=None, slu=None, rect=None):
        self.ixy = None  # 1 or 'x' if s=x,   2 or 'y' if s=y
        self.s = None    # array
        self.lower = None  # array of same length as s
        self.upper = None  # array of same length as s
        self.method = 0  # 0 for pw constant, 1 for pw linear
        self.ds = -1   # > 0 if s values are equally spaced
        if fname is not None:
            self.read(fname)
            if slu is not None:
                print('*** Warning: ignoring slu since fname also given')
            if rect is not None:
                print('*** Warning: ignoring rect since fname also given')
        elif slu is not None:
            self.s = slu[:,0]
            self.lower = slu[:,1]
            self.upper = slu[:,2]
            if rect is not None:
                print('*** Warning: ignoring rect since slu also given')
            assert np.diff(self.s).min() > 0, \
                '*** s must be monotonically increasing: \n  s =  %s' % self.s
            assert np.all(slu[:,1] <= slu[:,2]), \
                '*** values in L column of slu array must be <= U column'
        elif rect is not None:
            # define a simple rectangle:
            x1,x2,y1,y2 = rect
            self.s = np.array([x1,x2])
            self.lower = np.array([y1,y1])
            self.upper = np.array([y2,y2])
            self.ixy = 1
            self.method = 0
            self.ds = x2 - x1
        
    def bounding_box(self):
        if self.ixy in [1,'x']:
            x1 = self.s.min()
            x2 = self.s.max()
            y1 = self.lower.min()
            y2 = self.upper.max()
        elif self.ixy in [2,'y']:
            y1 = self.s.min()
            y2 = self.s.max()
            x1 = self.lower.min()
            x2 = self.upper.max()
        else:
            raise ValueError('Unrecognized attribute ixy = %s' % self.ixy)
        return [x1, x2, y1, y2]

    def slu(self):
        """
        Return npts by 3 array with columns (s, lower, upper)
        """
        assert np.diff(self.s).min() > 0, \
            '*** s must be monotonically increasing: \n  s =  %s' % self.s
        slu = np.vstack((self.s, self.lower, self.upper)).T
        return slu

            
    def vertices(self):
        """
        Return the vertices (x,y) of the polygon defined by this RuledRectangle.
        """
        
        assert np.diff(self.s).min() > 0, \
            '*** s must be monotonically increasing: \n  s =  %s' % self.s
        assert np.all(self.lower <= self.upper), \
            '*** values in lower array should be <= values in upper array'
            
        if self.method == 0:
            # stacked rectangles, requires doubling up points for vertices
            ss = [self.s[0]]
            for sk in self.s[1:-1]:
                ss += [sk,sk]
            ss = np.array(ss + [self.s[-1]])
            ss = np.hstack((ss, np.flipud(ss), self.s[0]))
            ll = []
            uu = []
            for k in range(len(self.s)-1):
                ll += [self.lower[k],self.lower[k]]
                uu += [self.upper[k],self.upper[k]]
            lu = np.hstack((ll, np.flipud(uu), self.lower[0]))
            
        elif self.method == 1:
            # points defined by s, lower, upper are connected by lines
            ss = np.hstack((self.s, np.flipud(self.s), self.s[0]))
            lu = np.hstack((self.lower, np.flipud(self.upper), self.lower[0]))
            
        if self.ixy in [1,'x']:
            x = ss
            y = lu
        elif self.ixy in [2,'y']:
            x = lu
            y = ss
        else:
            raise ValueError('Unrecognized attribute ixy = %s' % self.ixy)
        return x,y
            
            
    def mask_outside(self, X, Y):
        """
        Given 2d arrays X,Y, return a mask with the same shape with
        mask == True at points that are outside this RuledRectangle.
        So if Z is a data array defined at points X,Y then 
            ma.masked_array(Z, mask) 
        will be a masked array that can be used to plot only the values
        inside the Ruled Region.
        
        Works for self.method == 0 or 1, and also
              for self.s monotonically increasing or decreasing, 
                         but elsewhere we require increasing so check for that.
        
        """
        assert np.diff(self.s).min() > 0, \
            '*** s must be monotonically increasing: \n  s =  %s' % self.s
        assert np.all(self.lower <= self.upper), \
            '*** values in lower array should be <= values in upper array'
        
        transpose_arrays =  (X[0,0] == X[0,-1])
        if transpose_arrays:
            x = X[:,0]
            y = Y[0,:]
        else:
            x = X[0,:]
            y = Y[:,0]
        assert x[0] != x[-1], '*** Wrong orientation?'            
        
        def bracket(xy, s):
            """
            Given a point xy (an x or y value)
            and an array s that is monotone increasing or decreasing, 
            return indices (k1, k2) such that 
                s[k1] <= xy <= s[k2]
            If xy is outside the range of s, return (-1,-1)
            """
            from numpy import ma, argmin
            if xy < s.min() or xy > s.max():
                return -1,-1
            pos = ma.masked_where(xy-s < 0, xy-s)
            k1 = argmin(pos)
            pos = ma.masked_where(s-xy < 0, s-xy)
            k2 = argmin(pos)
            return k1,k2
            
        mask = np.empty((len(y),len(x)), dtype=bool)
        mask[:,:] = True
        if self.ixy in [1,'x']:
            # indices of x array that lie within s.min() to s.max():
            iin, = np.where(np.logical_and(self.s.min() <= x, x <= self.s.max()))
            
            for i in iin:
                # a column of array that might include points in the polygon
                xi = x[i]
                # determine k1,k2 such that s[k1] <= xi <= s[k2]:
                k1,k2 = bracket(xi, self.s)
                
                if k1 > -1:
                    sk1 = self.s[k1]
                    sk2 = self.s[k2]
                    if (self.method==0) or (sk1 >= xi):
                        ylower = self.lower[k1]
                        yupper = self.upper[k1]
                    else:
                        # method==1 and not at top, so linear interpolation:
                        alpha = (xi-sk1)/(sk2-sk1)
                        ylower = (1-alpha)*self.lower[k1] + \
                                     alpha*self.lower[k2]
                        yupper = (1-alpha)*self.upper[k1] + \
                                     alpha*self.upper[k2] 
                    # indices j for which ylower <= y[j] <= yupper, so inside:  
                    j, = np.where(np.logical_and(ylower <= y, y <= yupper))
                    mask[j,i] = False
                    
        elif self.ixy in [2,'y']:
            # indices of y array that lie within s.min() to s.max():
            jin, = np.where(np.logical_and(self.s.min() <= y, y <= self.s.max()))
            
            for j in jin:
                # a row of array that might include points in the polygon
                yj = y[j]
                # determine k1,k2 such that s[k1] <= yj <= s[k2]:
                k1,k2 = bracket(yj, self.s)
                
                if k1 > -1:
                    sk1 = self.s[k1]
                    sk2 = self.s[k2]
                    if (self.method==0) or (min(sk1,sk2) >= yj):
                        xlower = self.lower[k1]
                        xupper = self.upper[k1]
                    else:
                        # method==1 and not at top, so linear interpolation:
                        alpha = (yj-sk1)/(sk2-sk1)
                        xlower = (1-alpha)*self.lower[k1] + \
                                     alpha*self.lower[k2]
                        xupper = (1-alpha)*self.upper[k1] + \
                                     alpha*self.upper[k2]   
                    # indices i for which xlower <= x[i] <= xupper, so inside:
                    i, = np.where(np.logical_and(xlower <= x, x <= xupper))
                    mask[j,i] = False

        else:
            raise ValueError('Unrecognized attribute ixy = %s' % self.ixy)
                    
        if transpose_arrays:
            mask = mask.T
            
        return mask

                    
    def write(self, fname, verbose=False):
        slu = self.slu()
        ds = self.s[1:] - self.s[:-1]
        dss = ds.max() - ds.min()
        if dss < 1e-6*ds.max():
            self.ds = ds.max()
        else:
            self.ds = -1  # not uniformly spaced
            
        # if ixy is 'x' or 'y' replace by 1 or 2 for writing:
        if self.ixy in [1,'x']:
            ixyint = 1
        elif self.ixy in [2,'y']:
            ixyint = 2
        else:
            raise ValueError('Unrecognized attribute ixy = %s' % self.ixy)
            
        header = """\n%i   ixy\n%i   method\n%g    ds\n%i    nrules""" \
            % (ixyint,self.method,self.ds,len(self.s))
        np.savetxt(fname, slu,header=header,comments='',fmt='%.9f  ')
        
        if verbose:
            print("Created %s" % fname)

    def read(self, fname):
        lines = open(fname,'r').readlines()
        k = -1
        comments = True
        while comments:
            k += 1
            line = lines[k].strip()
            if (line != '') and (line[0] != '#'):
                comments = False
        self.ixy = int(line.split()[0])
        k += 1
        self.method = int(lines[k].split()[0])
        k += 1
        self.ds = float(lines[k].split()[0])
        k += 1
        self.nrules = int(lines[k].split()[0])
        slu = np.loadtxt(fname, skiprows=k+1)
        assert slu.shape[0] == self.nrules, '*** wrong shape'
        self.s = slu[:,0]
        self.lower = slu[:,1]
        self.upper = slu[:,2]
        assert np.all(slu[:,1] <= slu[:,2]), \
            '*** values in L column of slu array must be <= U column'
        
    def make_kml(self, fname='RuledRectangle.kml', name='RuledRectangle', 
                 color='00FFFF', width=2, verbose=False):
        from clawpack.geoclaw import kmltools
        x,y = self.vertices()
        kmltools.poly2kml((x,y), fname=fname, name=name, color=color, 
                          width=width, verbose=verbose)
        
        
def ruledrectangle_covering_selected_points(X, Y, pts_chosen, ixy, method=0,
                                            padding=0, verbose=True):
    """
    Given 2d arrays X,Y and an array pts_chosen of the same shape,
    returns a RuledRectangle that covers these points as compactly as possible.
    If method==0 then the RuledRectangle will be "stacked rectangles" that 
    cover all the cells indicated by pts_chosen 
    (assuming the X,Y points are cell centers on a uniform grid).
    
    If method==1, then the RuledRectangle will cover the cell centers but
    not all the full grid cells, since the edges will be lines connecting
    some cell centers.
    """
        
    if padding != 0:
        print('*** Warning, padding != 0 is not implemented!')
        
    if np.ndim(X) == 2:
        x = X[0,:]
        y = Y[:,0]
    else:
        x = X
        y = Y

    dx = x[1] - x[0]
    dy = y[1] - y[0]

    if ixy in [1,'x']:

        # Ruled rectangle with s = x:

        s = []
        lower = []
        upper = []
        for i in range(len(x)):
            if pts_chosen[:,i].sum() > 0:
                j = np.where(pts_chosen[:,i]==1)[0]
                j1 = j.min()
                j2 = j.max()
                s.append(x[i])
                lower.append(y[j1])
                upper.append(y[j2])
                
    elif ixy in [2,'y']:

        # Ruled rectangle with s = y:

        s = []
        lower = []
        upper = []

        for j in range(len(y)):
            if pts_chosen[j,:].sum() > 0:
                i = np.where(pts_chosen[j,:]==1)[0]
                i1 = i.min()
                i2 = i.max()
                s.append(y[j])
                lower.append(x[i1])
                upper.append(x[i2])
                
    else:
        raise(ValueError('Unrecognized value of ixy'))

    s = np.array(s)
    lower = np.array(lower)
    upper = np.array(upper)
    ds = s[1] - s[0]

    if method == 0:
        # extend so rectangles cover grid cells with centers at (x,y)
        if verbose:
            print('Extending rectangles to cover grid cells')
        if abs(dx - dy) > 1e-6:
            print('*** Warning, dx = %.8e not equal to dy = %.8e' \
                  % (dx, dy))
        lower = lower - 0.5*ds
        upper = upper + 0.5*ds
        s = s - 0.5*ds
        s = np.hstack((s, s[-1]+ds))
        lower = np.hstack((lower, lower[-1]))
        upper = np.hstack((upper, upper[-1]))
        
    rr = RuledRectangle()
    rr.ixy = ixy
    rr.s = s
    rr.lower = lower
    rr.upper = upper
    rr.method = method
    rr.ds = ds

    if verbose:
        rr_npts = int(np.ceil(np.sum(rr.upper - rr.lower) / ds))
        print('RuledRectangle covers %s grid points' % rr_npts)

    return rr




"""
Run several regression tests with different parameters.

Check the settings in the __main__ program to specify what is done.

To modify for a different app or set of tests, see the code labelled "Test 1", 
"Test 2", etc.
"""

from setrun_regression import setrun
from setplot import setplot
from clawpack.clawutil.runclaw import runclaw
from clawpack.visclaw.plotclaw import plotclaw
from clawpack.clawutil.compare_regression_tests import compare_regression_tests
import os,sys


def run_regression_tests(regression_dir="_regression_tests", \
            regression_output_files="all", \
            regression_plot_files="all", \
            make_new=True, run_tests=True, compare_results=True, \
            relocatable=False):

    if make_new:
        # Compile code from scratch to make sure up to date:
        os.system('make new')

    if run_tests:

        if os.path.exists(regression_dir):
            ans = input("Directory %s exists, ok to overwrite? " % regression_dir)
            if ans=='y':
                os.system('rm -rf %s' % regression_dir)
            else:
                print("*** Aborting regression tests")
                sys.exit()

        os.system('mkdir %s' % regression_dir)

        # -----------------------------------------------------------
        # Define the regression test runs:
        # -----------------------------------------------------------

        # initialize rundata using setrun but then change some things for each run:
        rundata = setrun()
        clawdata = rundata.clawdata
        amrdata = rundata.amrdata


        #--------------
        # Test 1:
        #--------------

        suffix = "_test1"
        outdir = regression_dir + "/_output" + suffix
        plotdir = regression_dir + "/_plots" + suffix

        amrdata.amr_levels_max = 1
        rundata.write()

        runclaw(xclawcmd = "xamr", outdir=outdir, print_git_status=True)
        plotclaw(outdir=outdir, plotdir=plotdir)

        #--------------
        # Test 2:
        #--------------
        amrdata.amr_levels_max = 2
        rundata.write()

        suffix = "_test2"
        outdir = regression_dir + "/_output" + suffix
        plotdir = regression_dir + "/_plots" + suffix

        runclaw(xclawcmd = "xamr", outdir=outdir, print_git_status=True)
        plotclaw(outdir=outdir, plotdir=plotdir)

        #--------------
        # Test 3:
        #--------------

        amrdata.amr_levels_max = 3
        rundata.write()

        suffix = "_test3"
        outdir = regression_dir + "/_output" + suffix
        plotdir = regression_dir + "/_plots" + suffix

        runclaw(xclawcmd = "xamr", outdir=outdir, print_git_status=True)
        plotclaw(outdir=outdir, plotdir=plotdir)

        # ----------------------------------------------------
        # End of test case definitions
        # ----------------------------------------------------

        print("Output and plots are in ", regression_dir)

    regression_ok = None

    if compare_results:
        regression_ok = compare_regression_tests(regression_dir,\
                        regression_output_files, regression_plot_files, \
                        relocatable=relocatable)
        if regression_ok:
            print("The specified regression files are identical in all")
            print("    directories checked")
        else:
            print("*** Some regression tests did not pass")

    return regression_ok

if __name__=="__main__":

    # Location for regression results:
    regression_dir = "_regression_tests"

    # Which output files to check in order to "pass" this test:
    # If these are identical for each test then run_regression_tests returns True.
    regression_output_files = ['fort.q0002','fort.gauge']
    regression_plot_files = ['frame0002fig0.png','gauge0001fig300.png']

    # Specify what to do:
    make_new = False        # run 'make new' to recompile everything?
    run_tests = False       # run all the tests?
    compare_results = True  # download archived results and compare?
    relocatable = True      # copy all image files so regression_dir can
                            #      be moved, e.g. posted for discussion

    run_regression_tests(regression_dir, regression_output_files, \
                regression_plot_files, \
                make_new, run_tests, compare_results, \
                relocatable=relocatable)



#!/usr/bin/env python
# encoding: utf-8

""" 
!-----------------------------------------------------------------------
! HDF5 and XDMF routines for creating output files for VisIt.
!-----------------------------------------------------------------------
"""
import os,sys
import h5py
from numpy import *

# Write a 2D HDF5 File for Rectilinear Mesh
def writeHDF52D(x,y,filename,out_dir):
    # Write H5 Grid Data
    h5_fname = out_dir + '/' + filename
    f = h5py.File(h5_fname,'w')
    dset = f.create_dataset("x",(x.size,),'f8')
    dset[...] = x
    dset = f.create_dataset("y",(y.size,),'f8')
    dset[...] = y
    f.close()
        
# Write a 3D HDF5 File for Rectilinear Mesh
def writeHDF53D(x,y,z,filename,out_dir):
    # Write H5 Grid Data
    h5_fname = out_dir + '/' + filename
    f = h5py.File(h5_fname,'w')
    dset = f.create_dataset("x",(x.size,),'f8')
    dset[...] = x
    dset = f.create_dataset("y",(y.size,),'f8')
    dset[...] = y
    dset = f.create_dataset("z",(z.size,),'f8')
    dset[...] = z
    f.close()

# Write a 3D HDF5 File for Curvilinear Mesh
def writeHDF53DS(x,y,z,names,filename,out_dir):
    # Write H5 Grid Data
    h5_fname = out_dir + '/' + filename
    f = h5py.File(h5_fname,'w')
    dset = f.create_dataset(names[0],(size(x,0),size(y,1),size(z,2),),'f8')
    dset[:] = x
    dset = f.create_dataset(names[1],(size(x,0),size(y,1),size(z,2),),'f8')
    dset[:] = y
    dset = f.create_dataset(names[2],(size(x,0),size(y,1),size(z,2),),'f8')
    dset[:] = z
    f.close()

# Write a XDMF (.xmf) File for 2D Rectilinear Mesh
def writeXDMF2D(t0,nx,ny,filename,mesh_name,out_dir,simTime):
    # Write XDMF (.xmf) file
    prefix = filename[:-7]
    xmfname = out_dir + '/' + filename[:-3] + '.xmf'
    f = open(xmfname,'w')
    f.write('<?xml version="1.0" ?> \n')
    f.write('<!DOCTYPE Xdmf SYSTEM "Xdmf.dtd" []> \n')
    f.write('<Xdmf Version="2.0"> \n')
    f.write(' <Domain> \n')
    f.write(' \n')
    f.write('   <!-- Logical Mesh  -->\n')
    f.write(' \n')
    f.write('   <Grid Name="mesh2D" GridType="Uniform">\n')
    f.write('     <Time TimeType="single" Value="'+str(simTime)+'"/>\n')
    f.write('     <Topology TopologyType="2DRectMesh" NumberOfElements="'+str(nx)+' '+str(ny)+'"/>\n')
    f.write('     <Geometry GeometryType="VxVy">\n')
    f.write('       <DataItem Dimensions="'+str(nx)+'" NumberType="Float" Precision="8" Format="HDF">\n')
    f.write('         '+mesh_name+':/x \n')
    f.write('       </DataItem>\n')
    f.write('       <DataItem Dimensions="'+str(ny)+'" NumberType="Float" Precision="8" Format="HDF">\n')
    f.write('         '+mesh_name+':/y \n')
    f.write('       </DataItem>\n')
    f.write('     </Geometry>\n')
    f.write('     <Attribute Name="'+prefix[:-1]+'" AttributeType="Scalar" Center="Node">\n')
    f.write('       <DataItem Dimensions="'+str(nx)+' '+str(ny)+'" NumberType="Float" Precision="8" Format="HDF">\n')
    f.write('         '+filename+':/'+prefix[:-1]+' \n')
    f.write('       </DataItem>\n')
    f.write('     </Attribute>\n')
    f.write('   </Grid>\n')
    f.write(' </Domain> \n')
    f.write('</Xdmf>')
    f.close()

# Write a XDMF (.xmf) File for 3D Rectilinear Mesh
def writeXDMF3D(nx,ny,nz,var_names,filename,mesh_name,out_dir,simTime):
    # Write XDMF (.xmf) file
    xmfname = out_dir + '/' + filename[:-3] + '.xmf'
    f = open(xmfname,'w')
    f.write('<?xml version="1.0" ?> \n')
    f.write('<!DOCTYPE Xdmf SYSTEM "Xdmf.dtd" []> \n')
    f.write('<Xdmf Version="2.0"> \n')
    f.write(' <Domain> \n')
    f.write(' \n')
    f.write('   <!-- Logical Mesh  -->\n')
    f.write(' \n')
    f.write('   <Grid Name="mesh" GridType="Uniform">\n')
    f.write('     <Time TimeType="single" Value="'+str(simTime)+'"/>\n')
    f.write('     <Topology TopologyType="3DRectMesh" NumberOfElements="'+str(nz)+' '+str(ny)+' '+str(nx)+'"/>\n')
    f.write('     <Geometry GeometryType="VxVyVz">\n')
    f.write('       <DataItem Dimensions="'+str(nx)+'" NumberType="Float" Precision="8" Format="HDF">\n')
    f.write('         '+mesh_name+':/x \n')
    f.write('       </DataItem>\n')
    f.write('       <DataItem Dimensions="'+str(ny)+'" NumberType="Float" Precision="8" Format="HDF">\n')
    f.write('         '+mesh_name+':/y \n')
    f.write('       </DataItem>\n')
    f.write('       <DataItem Dimensions="'+str(nz)+'" NumberType="Float" Precision="8" Format="HDF">\n')
    f.write('         '+mesh_name+':/z \n')
    f.write('       </DataItem>\n')
    f.write('     </Geometry>\n')
    for var_name in var_names:
        f.write('     <Attribute Name="'+var_name+'" AttributeType="Scalar" Center="Node">\n')
        f.write('       <DataItem Dimensions="'+str(nz)+' '+str(ny)+' '+str(nx)+'" NumberType="Float" Precision="8" Format="HDF">\n')
        f.write('         '+filename+':/'+var_name+' \n')
        f.write('       </DataItem>\n')
        f.write('     </Attribute>\n')
    f.write('   </Grid>\n')
    f.write(' </Domain> \n')
    f.write('</Xdmf>')
    f.close()

# Write a XDMF (.xmf) File for 3D Curvilinear Mesh
def writeXDMF3DSMesh(nx,ny,nz,var_names,filename,mesh_name,out_dir,simTime):
    # Write XDMF (.xmf) file
    xmfname = out_dir + '/' + filename[:-3] + '.xmf'
    f = open(xmfname,'w')
    f.write('<?xml version="1.0" ?> \n')
    f.write('<!DOCTYPE Xdmf SYSTEM "Xdmf.dtd" []> \n')
    f.write('<Xdmf Version="2.0"> \n')
    f.write(' <Domain> \n')
    f.write(' \n')
    f.write('   <!-- Logical Mesh  -->\n')
    f.write(' \n')
    f.write('   <Grid Name="mesh" GridType="Uniform">\n')
    f.write('     <Time TimeType="single" Value="'+str(simTime)+'"/>\n')
    f.write('     <Topology TopologyType="3DSMesh" NumberOfElements="'+str(nx)+' '+str(ny)+' '+str(nz)+'"/>\n')
    f.write('     <Geometry GeometryType="X_Y_Z">\n')
    f.write('       <DataItem Dimensions="'+str(nx)+' '+str(ny)+' '+str(nz)+'" NumberType="Float" Precision="8" Format="HDF">\n')
    f.write('         '+mesh_name+':/x \n')
    f.write('       </DataItem>\n')
    f.write('       <DataItem Dimensions="'+str(nx)+' '+str(ny)+' '+str(nz)+'" NumberType="Float" Precision="8" Format="HDF">\n')
    f.write('         '+mesh_name+':/y \n')
    f.write('       </DataItem>\n')
    f.write('       <DataItem Dimensions="'+str(nx)+' '+str(ny)+' '+str(nz)+'" NumberType="Float" Precision="8" Format="HDF">\n')
    f.write('         '+mesh_name+':/z \n')
    f.write('       </DataItem>\n')
    f.write('     </Geometry>\n')
    for var_name in var_names:
        f.write('     <Attribute Name="'+var_name+'" AttributeType="Scalar" Center="Node">\n')
        #f.write('       <DataItem Dimensions="'+str(nz*ny*nx)+'" NumberType="Float" Precision="8" Format="HDF">\n')
        f.write('       <DataItem Dimensions="'+str(nx)+' '+str(ny)+' '+str(nz)+'" NumberType="Float" Precision="8" Format="HDF">\n')
        f.write('         '+filename+':/'+var_name+' \n')
        f.write('       </DataItem>\n')
        f.write('     </Attribute>\n')
    f.write('   </Grid>\n')
    f.write(' </Domain> \n')
    f.write('</Xdmf>')
    f.close()

# Write a XDMF (.xmf) File for 3D Rectilinear Mesh with HyperSlab Data 
def writeXDMF3DSMeshHyper(nx,ny,nz,var_names,filename,mesh_name,out_dir,simTime):
    # Write XDMF (.xmf) file
    xmfname = out_dir + '/' + filename[:-4] + '.xmf'
    f = open(xmfname,'w')
    f.write('<?xml version="1.0" ?> \n')
    f.write('<!DOCTYPE Xdmf SYSTEM "Xdmf.dtd" []> \n')
    f.write('<Xdmf Version="2.0"> \n')
    f.write(' <Domain> \n')
    f.write(' \n')
    f.write('   <!-- Logical Mesh  -->\n')
    f.write(' \n')
    f.write('   <Grid Name="mesh" GridType="Uniform">\n')
    f.write('     <Time TimeType="single" Value="'+str(simTime)+'"/>\n')
    f.write('     <Topology TopologyType="3DSMesh" NumberOfElements="'+str(nx)+' '+str(ny)+' '+str(nz)+'"/>\n')
    f.write('     <Geometry GeometryType="X_Y_Z">\n')
    f.write('       <DataItem Dimensions="'+str(nx)+' '+str(ny)+' '+str(nz)+'" NumberType="Float" Precision="8" Format="HDF">\n')
    f.write('         '+mesh_name+':/x \n')
    f.write('       </DataItem>\n')
    f.write('       <DataItem Dimensions="'+str(nx)+' '+str(ny)+' '+str(nz)+'" NumberType="Float" Precision="8" Format="HDF">\n')
    f.write('         '+mesh_name+':/y \n')
    f.write('       </DataItem>\n')
    f.write('       <DataItem Dimensions="'+str(nx)+' '+str(ny)+' '+str(nz)+'" NumberType="Float" Precision="8" Format="HDF">\n')
    f.write('         '+mesh_name+':/z \n')
    f.write('       </DataItem>\n')
    f.write('     </Geometry>\n')
    ieqn = 0
    neqn = size(var_names)
    for var_name in var_names:
        f.write('     <Attribute Name="'+var_name+'" AttributeType="Scalar" Center="Node">\n')
        f.write('       <DataItem ItemType="HyperSlab" Dimensions="1 '+str(nx)+' '+str(ny)+' '+str(nz)+'" Type="HyperSlab">\n')
        f.write('         <DataItem Dimensions="3 4" Format="XML">\n')
        f.write('           '+str(ieqn)+' '+str(0)+' '+str(0)+' '+str(0)+' \n')
        f.write('           '+str(1)+' '+str(1)+' '+str(1)+' '+str(1)+' \n')
        f.write('           '+str(1)+' '+str(nx)+' '+str(ny)+' '+str(nz)+' \n')
        f.write('         </DataItem>\n')
        f.write('         <DataItem Dimensions="'+str(neqn)+' '+str(nx)+' '+str(ny)+' '+str(nz)+'" NumberType="Float" Precision="8" Format="HDF">\n')
        f.write('           '+filename+':/patch1/q\n')
        f.write('         </DataItem>\n')
        f.write('       </DataItem>\n')
        f.write('     </Attribute>\n')
        ieqn+=1
    f.write('   </Grid>\n')
    f.write(' </Domain> \n')
    f.write('</Xdmf>')
    f.close()



"""
Run GeoClaw with pure shallow water equations and with several choices
of Boussinesq equations, and then plot gauge results compared to experiment.

"""
from pylab import *
import os

from clawpack.clawutil.runclaw import runclaw
#from clawpack.visclaw.frametools import plotframe
#from clawpack.visclaw.data import ClawPlotData
import compare_gauges

import setrun

outdir_sgn = '_output_sgn'
print('outdir_bouss = ',outdir_sgn)

outdir_ms = '_output_ms'
print('outdir_ms = ',outdir_ms)

outdir_swe = '_output_swe'
print('outdir_swe = ',outdir_swe)

run_code = True  # set to False if output already exists

if run_code:
    # create executable and .data files:
    os.system('make .exe')
    rundata = setrun.setrun()

    # Boussinesq, MS:
    rundata.bouss_data.bouss_equations = 1
    rundata.write()
    runclaw(xclawcmd='xgeo',outdir=outdir_ms)   # run clawpack code

    # Boussinesq, SGN:
    rundata.bouss_data.bouss_equations = 2
    rundata.write()
    runclaw(xclawcmd='xgeo',outdir=outdir_sgn)   # run clawpack code
    
    # Shallow water equations:
    rundata.bouss_data.bouss_equations = 0
    rundata.write()
    runclaw(xclawcmd='xgeo',outdir=outdir_swe)   # run clawpack code
    

outdirs=[('_output_swe', 'SWE', 'k'), \
         ('_output_ms', 'MS', 'b'), \
         ('_output_sgn', 'SGN','g')]

compare_gauges.plot_gauges(outdirs, fname_figure='GaugeComparison_BoussSWE.png')




from __future__ import absolute_import
from __future__ import print_function

import os

import numpy
import matplotlib.pyplot as plt
import datetime

import clawpack.visclaw.colormaps as colormap
import clawpack.visclaw.gaugetools as gaugetools
import clawpack.clawutil.data as clawutil
import clawpack.amrclaw.data as amrclaw
import clawpack.geoclaw.data as geodata


import clawpack.geoclaw.surge.plot as surgeplot

try:
    from setplotfg import setplotfg
except:
    setplotfg = None


def setplot(plotdata=None):
    """"""

    if plotdata is None:
        from clawpack.visclaw.data import ClawPlotData
        plotdata = ClawPlotData()

    # clear any old figures,axes,items data
    plotdata.clearfigures()
    plotdata.format = 'ascii'

    # Load data from output
    clawdata = clawutil.ClawInputData(2)
    clawdata.read(os.path.join(plotdata.outdir, 'claw.data'))
    physics = geodata.GeoClawData()
    physics.read(os.path.join(plotdata.outdir, 'geoclaw.data'))
    surge_data = geodata.SurgeData()
    surge_data.read(os.path.join(plotdata.outdir, 'surge.data'))
    friction_data = geodata.FrictionData()
    friction_data.read(os.path.join(plotdata.outdir, 'friction.data'))

    # Load storm track
    track = surgeplot.track_data(os.path.join(plotdata.outdir, 'fort.track'))

    # Set afteraxes function
    def surge_afteraxes(cd):
        surgeplot.surge_afteraxes(cd, track, plot_direction=False,
                                             kwargs={"markersize": 4})

    # Color limits
    surface_limits = [-5.0, 5.0]
    speed_limits = [0.0, 3.0]
    wind_limits = [0, 64]
    pressure_limits = [935, 1013]
    friction_bounds = [0.01, 0.04]

    def friction_after_axes(cd):
        plt.title(r"Manning's $n$ Coefficient")

    # ==========================================================================
    #   Plot specifications
    # ==========================================================================
    regions = {"Gulf": {"xlimits": (clawdata.lower[0], clawdata.upper[0]),
                        "ylimits": (clawdata.lower[1], clawdata.upper[1]),
                        "figsize": (6.4, 4.8)},
               "LaTex Shelf": {"xlimits": (-97.5, -88.5),
                               "ylimits": (27.5, 30.5),
                               "figsize": (8, 2.7)}}

    for (name, region_dict) in regions.items():

        # Surface Figure
        plotfigure = plotdata.new_plotfigure(name="Surface - %s" % name)
        plotfigure.kwargs = {"figsize": region_dict['figsize']}
        plotaxes = plotfigure.new_plotaxes()
        plotaxes.title = "Surface"
        plotaxes.xlimits = region_dict["xlimits"]
        plotaxes.ylimits = region_dict["ylimits"]
        plotaxes.afteraxes = surge_afteraxes

        surgeplot.add_surface_elevation(plotaxes, bounds=surface_limits)
        surgeplot.add_land(plotaxes, bounds=[0.0, 20.0])
        plotaxes.plotitem_dict['surface'].amr_patchedges_show = [0] * 10
        plotaxes.plotitem_dict['land'].amr_patchedges_show = [0] * 10

        # Speed Figure
        plotfigure = plotdata.new_plotfigure(name="Currents - %s" % name)
        plotfigure.kwargs = {"figsize": region_dict['figsize']}
        plotaxes = plotfigure.new_plotaxes()
        plotaxes.title = "Currents"
        plotaxes.xlimits = region_dict["xlimits"]
        plotaxes.ylimits = region_dict["ylimits"]
        plotaxes.afteraxes = surge_afteraxes

        surgeplot.add_speed(plotaxes, bounds=speed_limits)
        surgeplot.add_land(plotaxes, bounds=[0.0, 20.0])
        plotaxes.plotitem_dict['speed'].amr_patchedges_show = [0] * 10
        plotaxes.plotitem_dict['land'].amr_patchedges_show = [0] * 10
    #
    # Friction field
    #
    plotfigure = plotdata.new_plotfigure(name='Friction')
    plotfigure.show = friction_data.variable_friction and True

    plotaxes = plotfigure.new_plotaxes()
    plotaxes.xlimits = regions['Gulf']['xlimits']
    plotaxes.ylimits = regions['Gulf']['ylimits']
    # plotaxes.title = "Manning's N Coefficient"
    plotaxes.afteraxes = friction_after_axes
    plotaxes.scaled = True

    surgeplot.add_friction(plotaxes, bounds=friction_bounds, shrink=0.9)
    plotaxes.plotitem_dict['friction'].amr_patchedges_show = [0] * 10
    plotaxes.plotitem_dict['friction'].colorbar_label = "$n$"

    #
    #  Hurricane Forcing fields
    #
    # Pressure field
    plotfigure = plotdata.new_plotfigure(name='Pressure')
    plotfigure.show = surge_data.pressure_forcing and True

    plotaxes = plotfigure.new_plotaxes()
    plotaxes.xlimits = regions['Gulf']['xlimits']
    plotaxes.ylimits = regions['Gulf']['ylimits']
    plotaxes.title = "Pressure Field"
    plotaxes.afteraxes = surge_afteraxes
    plotaxes.scaled = True
    surgeplot.add_pressure(plotaxes, bounds=pressure_limits)
    surgeplot.add_land(plotaxes, bounds=[0.0, 20.0])

    # Wind field
    plotfigure = plotdata.new_plotfigure(name='Wind Speed')
    plotfigure.show = surge_data.wind_forcing and True

    plotaxes = plotfigure.new_plotaxes()
    plotaxes.xlimits = regions['Gulf']['xlimits']
    plotaxes.ylimits = regions['Gulf']['ylimits']
    plotaxes.title = "Wind Field"
    plotaxes.afteraxes = surge_afteraxes
    plotaxes.scaled = True
    surgeplot.add_wind(plotaxes, bounds=wind_limits)
    surgeplot.add_land(plotaxes, bounds=[0.0, 20.0])

    # ========================================================================
    #  Figures for gauges
    # ========================================================================
    plotfigure = plotdata.new_plotfigure(name='Gauge Surfaces', figno=300,
                                         type='each_gauge')
    plotfigure.show = True
    plotfigure.clf_each_gauge = True

    plotaxes = plotfigure.new_plotaxes()
    plotaxes.time_scale = 1 / (24 * 60**2)
    plotaxes.grid = True
    plotaxes.xlimits = 'auto'
    plotaxes.ylimits = 'auto'
    plotaxes.title = "Surface"
    plotaxes.ylabel = "Surface (m)"
    plotaxes.time_label = "Days relative to landfall"
    
    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    plotitem.plot_var = surgeplot.gauge_surface
    # Plot red area if gauge is dry
    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    plotitem.plot_var = surgeplot.gauge_dry_regions
    plotitem.kwargs = {"color":'lightcoral', "linewidth":5}
    
    #  Gauge Location Plot
    def gauge_location_afteraxes(cd):
        plt.subplots_adjust(left=0.12, bottom=0.06, right=0.97, top=0.97)
        surge_afteraxes(cd)
        gaugetools.plot_gauge_locations(cd.plotdata, gaugenos='all',
                                        format_string='ko', add_labels=True)

    plotfigure = plotdata.new_plotfigure(name="Gauge Locations")
    plotfigure.show = True

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.title = 'Gauge Locations'
    plotaxes.scaled = True
    plotaxes.xlimits = [-95.5, -94]
    plotaxes.ylimits = [29.0, 30.0]
    plotaxes.afteraxes = gauge_location_afteraxes
    surgeplot.add_surface_elevation(plotaxes, bounds=surface_limits)
    surgeplot.add_land(plotaxes, bounds=[0.0, 20.0])
    plotaxes.plotitem_dict['surface'].amr_patchedges_show = [0] * 10
    plotaxes.plotitem_dict['land'].amr_patchedges_show = [0] * 10

    # -----------------------------------------
    # Parameters used only when creating html and/or latex hardcopy
    # e.g., via pyclaw.plotters.frametools.printframes:

    plotdata.printfigs = True                # print figures
    plotdata.print_format = 'png'            # file format
    plotdata.print_framenos = 'all'          # list of frames to print
    plotdata.print_gaugenos = [1, 2, 3, 4]   # list of gauges to print
    plotdata.print_fignos = 'all'            # list of figures to print
    plotdata.html = True                     # create html files of plots?
    plotdata.latex = True                    # create latex file of plots?
    plotdata.latex_figsperline = 2           # layout of plots
    plotdata.latex_framesperline = 1         # layout of plots
    plotdata.latex_makepdf = False           # also run pdflatex?
    plotdata.parallel = True                 # parallel plotting

    return plotdata
